<?php
if (!defined('PLX_ROOT')) exit;

define('TABPAGER1', "\t\t\t\t\t");
define('TABPAGER2', "\t\t\t");

if (! defined('JQUERY_LIB'))
	define('JQUERY_LIB', 'jquery-1.11.3.min.js');
if (! defined('JQUERY_SRC')) {
	define('JQUERY_SRC', '//code.jquery.com/'.JQUERY_LIB);
}

class myPager extends plxPlugin {

	private $usingShowArticlesHook = false;
	private $direct_page = false;
	private $articles_navigator = true;

	public function __construct($default_lang) {

		# appel du constructeur de la classe plxPlugin (obligatoire)
		parent::__construct($default_lang);

		# droits pour accéder à la page config.php du plugin
		$this->setConfigProfil(PROFIL_ADMIN);

		$this->articles_navigator = (intval($this->getParam('articles_navigator')) > 0);

		$this->addHook('AdminTopEndHead', 'AdminTopEndHead');
		$this->addHook('AdminIndexPagination', 'AdminIndexPagination');
		$this->addHook('AdminCommentsPagination', 'AdminCommentsPagination');

		// pour des pages de chapôs d'articles
		$this->addHook('plxShowPagination', 'plxShowPagination');
		// pour une page d'article, ajoute le navigateur d'articles
		$this->addHook('IndexBegin', 'IndexBegin');
		$this->addHook('ThemeEndHead', 'ThemeEndHead');
		if ($this->articles_navigator) {
			$this->addHook('ThemeEndBody', 'ThemeEndBody');
			// Hooks créés pour ce plugin :
			// pour la page d'un article seul
			$this->addHook('showArticlesBar', 'showArticlesBar');
			// pour un groupe de pages statiques
			$this->addHook('plxShowStaticPagination', 'plxShowStaticPagination');
		}
	}

	// some useful functions to get a URL from page number

	// used by core/admin/index.php
	private function getUrl4Index($pageNumber) {
		return 'index.php?page='.$pageNumber;
	}

	// used by core/admin/index.php
	private function getUrl4Comments($pageNumber) {
		return 'comments.php?page='.$pageNumber;
	}

	// used by plxShowPagination hook
	private function urlPage($numero) {
		global $plxMotor;

		$result = $plxMotor->get;
		if ($result) { // On vient d'une page autre que celle de l'accueil index.php
			if (preg_match('/(\/?page[0-9]+)$/', $result, $matches))
				$result  = str_replace($matches[1], '', $result);
			if ($numero > 1) {
				if (strlen($result) > 0)
					 // Affichage d'une partie des articles selon que mode est dans (categorie, tags, archives)
					$result .= '/page';
				else
					// Affichage de tous les articles
					$result = 'page';
				$result .= $numero;
			}
		}
		elseif ($numero > 1)
			$result = 'page'.$numero;
		if ($result)
			// On affiche autre chose que la page d'accueil index.php
			$result = '?'.$result;
		return $plxMotor->urlRewrite($result);
	}

	// used by plxShowStaticPagination hook
	private function urlStaticPage($numero) {
		global $plxShow;

		// $numero starts at 1, $this->staticPagesGroup starts at 0
		$id = $this->staticPagesGroup[$numero - 1];
		$url = $plxShow->plxMotor->aStats[$id]['url'];
		// '?static'.intval($k).'/'.$v['url']
		$result = '?static'.$id.'/'.$url;
		return $plxShow->plxMotor->urlRewrite($result);
	}

	private function pluginRoot() {
		global $plxShow;

		if (isset($plxShow)) {
			return $plxShow->plxMotor->racine.$plxShow->plxMotor->aConf['racine_plugins'].__CLASS__.'/';
		} else {
			global $plxAdmin;
			if (isset($plxAdmin)) {
				return $plxAdmin->racine.$plxAdmin->aConf['racine_plugins'].__CLASS__.'/';
			} else {
				return false;
			}
		}
	}

	private function add_scripts() {
		if (! defined('JQUERY_LOADED')) { ?>
	<script type="text/javascript">
		if (typeof jQuery === 'undefined')
			document.write('<scr'+'ipt type="text/javascript" src'+'="<?php echo JQUERY_SRC; ?>"></scr'+'ipt>');
	</script> <!-- myPager -->
<?php
			define('JQUERY_LOADED', true);
		} ?>
	<script type="text/javascript" src="<?php echo $this->pluginRoot().__CLASS__; ?>.js"></script>
	<script type="text/javascript" src="<?php echo $this->pluginRoot(); ?>hammer.min.js"></script>
	<script type="text/javascript">
		<!--
		var msgMyPager = ['<?php echo $this->getLang('L_MYPAGER_FIRST_PAGE'); ?>', '<?php echo $this->getLang('L_MYPAGER_LAST_PAGE'); ?>'];
		// -->
	</script>
<?php
	}

	// for mode == home or core/admin/
	private function buildButtons($pageCC, $totalPages, $getUrl) {
		define('ACTIVE_PAGE', ' class="activePage"');
		// define('ACTIVE_PAGE', ' id="p_current"');

		$n = intval($this->getParam('delta'));
		$delta = ($n > 1) ? $n : 3; // nbre de boutons supplémentaires de chaque côté du bouton central, excepté les 2 boutons extrêmes
		$result = '';
		if ($totalPages > 1) {
			$result = (intval($this->getParam('display_page')) > 0) ? $this->lang('L_MYPAGER_PAGE')."\n" : '';
			$active = ($pageCC == 1) ? ACTIVE_PAGE : '';
			$n = intval($this->getParam('buttons'));
			if ($pageCC > 1) {
				$prevPage = $pageCC - 1;
				$onclick = '';
			}
			else {
				$prevPage = 1;
				$onclick = ' onclick="alert(\''.$this->getLang('L_MYPAGER_FIRST_PAGE').'\'); return false;"';
			}
			$buttons = (intval($this->getParam('buttons')) > 0);
			if ($buttons)
				$result .= TABPAGER1.'<a href="'.call_user_func_array(array(__CLASS__, $getUrl), array(1)).'"'.$active.$onclick.'>&lt;&lt;</a>'."\n"; // page n°1
			$result .= TABPAGER1.'<a href="'.call_user_func_array(array(__CLASS__, $getUrl), array($prevPage)).'"'.$onclick.'>&lt;</a>'."\n"; // recule d'une page
			$result .= TABPAGER1.'<a href="'.call_user_func_array(array(__CLASS__, $getUrl), array(1)).'"'.$active.'>1</a>'."\n"; // page n°1
			if ($totalPages > 2) { // pages intermédiaires
				$result .= TABPAGER1.'<span>'."\n";
				// ------ début boutons centraux
				if ($pageCC < 2 + $delta) {
					$iMin = 2;
					$iMax = ($totalPages > 2 * $delta + 3) ? $iMin + 2 * $delta + 1 : $totalPages;
				}
				elseif ($pageCC > $totalPages - $delta - 1) {
					$iMax = $totalPages ;
					$iMin = ($iMax > 2 * $delta + 2) ? $iMax - 2 * $delta - 1: 2;
				}
				else {
					$iMax = $pageCC + $delta + 1;
					$iMin = $pageCC - $delta;
				}
				for ($i = $iMin; $i < $iMax; $i++) {
					$active = ($pageCC == $i) ? ACTIVE_PAGE : '';
					$result .= TABPAGER1."\t".'<a href="'.call_user_func_array(array(__CLASS__, $getUrl), array($i)).'"'.$active.'>'.$i.'</a>'."\n";
				}
				// ------ fin boutons centraux
				$result .= TABPAGER1.'</span>'."\n";
			}
			$active = ($pageCC == $totalPages) ? ACTIVE_PAGE : '';
			// Mark the last page link for retrieving url to direct access at any page except page #1
			$result .= TABPAGER1.'<a id="last-page" href="'.call_user_func_array(array(__CLASS__, $getUrl), array($totalPages)).'"'.$active.'>'.$totalPages."</a>\n"; // last page
			if ($pageCC < $totalPages) {
				$nextPage = $pageCC + 1;
				$onclick = '';
			}
			else {
				$nextPage = $totalPages;
				$onclick = ' onclick="alert(\''.$this->getLang('L_MYPAGER_LAST_PAGE').'\'); return false;"';
			}
			$result .= TABPAGER1.'<a href="'.call_user_func_array(array(__CLASS__, $getUrl), array($nextPage)).'"'.$onclick.'>&gt;</a>'."\n"; // one page forward
			if ($buttons) // somebody request a ">>" button to direct access at the last page
				$result .= TABPAGER1.'<a href="'.call_user_func_array(array(__CLASS__, $getUrl), array($totalPages)).'"'.$active.$onclick.'>&gt;&gt;</a>'."\n"; // last page
		}
		return $result;
	}

	// add css rules
	public function AdminTopEndHead($params) {
		global $plxAdmin;
		global $arts;

		$scriptName = basename($_SERVER['SCRIPT_NAME'], '.php');
		if (in_array($scriptName, array('parametres_plugin', 'index'))) { ?>
	<link rel="stylesheet" type="text/css" href="<?php echo $this->pluginRoot().__CLASS__; ?>.css" media="screen" />
<?php
		    switch ($scriptName) {
                case 'index' :
                    if ($plxAdmin->page > 1) { ?>
	<link rel="prev" href="<?php echo $this->getUrl4Index($plxAdmin->page - 1); ?>" />
<?php				}
					$pagesCnt = ceil($plxAdmin->plxGlob_arts->count / $plxAdmin->bypage);
                    if ($plxAdmin->page < $pagesCnt) { ?>
	<link rel="next" href="<?php echo $this->getUrl4Index($plxAdmin->page + 1); ?>" />
<?php				}
                    break;
                case 'parametres_plugin' :
                    break;
            }
		$this->add_scripts();
        }
	}

	public function AdminIndexPagination() {
		global $arts, $nbArtPagination, $plxAdmin;

		$totalPages = ceil($nbArtPagination / $plxAdmin->bypage);
		$pageCC = $plxAdmin->page;
		$arts = false;
		echo $this->buildButtons($pageCC, $totalPages, 'getUrl4Index');
	}

	public function AdminCommentsPagination() {
		global $coms, $nbComPagination, $plxAdmin;
		$coms = false;
		$totalPages = ceil($nbComPagination / $plxAdmin->aConf['bypage_admin_coms']);
		$pageCC = $plxAdmin->page;
		echo $this->buildButtons($pageCC, $totalPages, 'getUrl4Comments');
	}

	// Hook called in home, tags, categorie, archives mode for displaying buttons under the list of articles
	public function plxShowPagination($params) {
		// appelé par home.php, categorie.php, tags.php, archives.php dans /themes/mon_theme/ via $plxShow->pagination()
		global $plxMotor;

		$byPage = $plxMotor->bypage;
		$articlesCount = $plxMotor->plxGlob_arts->count;
		$totalPages = ceil($articlesCount / $byPage);
		if ($totalPages > 1) {
			$pageCC = $plxMotor->page;
			$code = $this->buildButtons($pageCC, $totalPages, 'urlPage');
			if ($this->direct_page) {
				$code .=
					TABPAGER1.'<form onsubmit="return myPagerOnsubmit(this);">'."\n\t".
					TABPAGER1.'<label for="id_direct_page">'.$this->getLang('L_MYPAGER_DIRECT_PAGE_LABEL').'</label>'."\n\t".
					TABPAGER1.'<input id="id_direct_page" name="direct_page" size="2" maxlength="3" />'."\n\t".
					TABPAGER1.'<input type="submit" value="Ok" />'."\n".
					TABPAGER1."</form>\n";
			}
		}
		$code .= '<?php return true; ?>';
		echo $code;
	}

	// Hook called  in static pages
	public function plxShowStaticPagination($params) {
		global $plxShow;
		if ($this->staticPagesGroup) {
			$pageCC = $this->staticPageCC;
			$totalPages = count($this->staticPagesGroup); ?>
			<div id="pagination">
<?php	echo $this->buildButtons($pageCC, $totalPages, 'urlStaticPage'); ?>
			</div>
<?php		}
	}

	/*****************************************************************************************************/
	/* The following functions are used to display a navigator for articles when an article is displayed */
	/*****************************************************************************************************/

	// Store mode and cible when we are in home, categorie, archives or tags mode for using when we are in article mode
	// So, in article mode, we can retrieve the previous listing of articles.
	public function IndexBegin($params) {
		global $plxMotor;
		global $plxShow;  // required by $mode == 'archives'

		if ($plxMotor->mode == 'article') {
			// we recall the mode and get these articles for the previous listing
			$previousMode = $_SESSION['myPagerMode'];
			$previousCible = $_SESSION['myPagerCible'];
			if ($previousMode != 'categorie')
				$cats = '(?:home,|pin,|\d{3},)*(?:'.$plxMotor->activeCats.')(?:,\d{3})*';
			switch ($previousMode) {
				case 'categorie' :
					$this->caption = $plxMotor->aCats[$previousCible]['name'];
					$motif = '/^\d{4}\.(?:home,|pin,|\d{3},)*'.$previousCible.'(?:,\d{3})*\.\d{3}\.\d{12}\.[\w-]+\.xml$/i';
					break;
				case 'tags' :
					$this->caption = $previousCible;
					$articlesId = array_keys(array_filter($plxMotor->aTags, array(__CLASS__, 'filter_tags')));
					if (count($articlesId) > 0)
						$motif = '/(?:'.implode('|', $articlesId).')\.'.$cats.'\.\d{3}\.\d{12}\.[\w-]+\.xml$/i';
					break;
				case 'archives' : // articles for this archive
					$this->caption = ucfirst(plxDate::formatDate($previousCible, '#month #num_year(4)'));
					$cible = $previousCible.'\d{'.(12 - strlen($previousCible)).'}';
					$motif = '/^\d{4}\.'.$cats.'\.\d{3}\.'.$cible.'\.[\w-]+\.xml$/i';
					break;
				case 'author' : // not use
					$this->caption = 'author';
					$motif = '/^\d{4}\.'.$cats.'\.'.$previousCible.'\.\d{12}\.[\w-]+\.xml$/i';
					break;
				case 'home' : // all articles
					$this->caption = 'tous';
					$motif = '/^\d{4}\.'.$cats.'\.\d{3}\.\d{12}\.[\w-]+\.xml$/i';
					break;
				default :
					$this->caption = '';
					$motif = false;
			}
			if ($motif != false) {
				$this->articles = $plxMotor->plxGlob_arts->query($motif , 'art', 'rsort');
			} else {
				$this->articles = false;
			}
		}
		else if ($plxMotor->mode == 'static') {
			// on recherche les pages statiques d'un même groupe
			$cible = $plxMotor->cible;
			$group = trim($plxMotor->aStats[$cible]['group']);
			if ($group != '') {
				$this->staticPagesGroup = array();
				foreach ($plxMotor->aStats as $id=>$infos) {
					if ((trim($infos['active'] == 1) and (trim($infos['group']) == $group))) {
						$this->staticPagesGroup[] = $id;
						if ($cible == $id)
							$this->staticPageCC = count($this->staticPagesGroup);
					}
				}
			} else
				$this->staticPagesGroup = false;
		}
		else {
			// we store the mode for listing articles
			$_SESSION['myPagerMode'] = $plxMotor->mode;
			$_SESSION['myPagerCible'] = ($plxMotor->mode == 'tags') ? $plxMotor->cibleName : $plxMotor->cible;
		}
	}

	// stop playing! Retrieve mode and cible and catch the right articles.
	private function filter_tags($element) {
		return (preg_match('/\b('.$this->caption.')\b/i', $element['tags']) > 0);
	}

	private function urlArticle($filename) {
		global $plxMotor;

		$n = preg_match('/^(\d{4})\..*\.([\w-]+)\.xml$/i', $filename, $matches);
		$id = intval($matches[1]);
		$url = $matches[2];
		$result = $plxMotor->urlRewrite('?article'.$id.'/'.$url);
		return $result;
	}

	private function getArticle($filename) {
		global $plxMotor;

		return $article = $plxMotor->parseArticle(PLX_ROOT.$plxMotor->aConf['racine_articles'].$filename);
	}

	private function buildLinkArticle($filename, $caption=false) {
		$n = ($caption === false) ? '' : $caption;
		if ($filename) {
			$article = $this->getArticle($filename);
			$extra =  ' title="'.$article['title'].'"';
			$url = $this->urlArticle($filename);
			$alert = '';
		} else {
			$extra = ' onclick="alert(\'plus de page\'); return false;"';
			$url = '#';
		}
		return '<a href="'.$url.'"'.$extra.'>'.$n.'</a>'."\n";
	}

	// for mode == article. build the user interface after catching the right articles with getArticles() function
	private function articlesNavigator() {
		global $plxShow;

		if ($plxShow->mode() == 'article') {
			$values = array_values($this->articles);
			$articlesCount = count($values);
			// build a list of selected articles (categorie, tags, archives, ..)
			$buf = <<< BUF
			<select onchange="return callArticle(this.value);">\n
BUF;
			$n = false;
			for ($i = 0; $i<count($values); $i++) {
				$v = $values[$i];
				if (substr($v, 0, 4) == $plxShow->plxMotor->cible) {
					$n = $i;
					$selected = ' selected';
				}
				else
					$selected = '';
				$value = $this->urlArticle($v);
				$article = $this->getArticle($v);
				$caption = $article['title'];
				$buf .= <<< BUF
				<option value="$value"$selected>$caption</option>\n
BUF;
			}
			$buf .= <<< BUF
			</select>\n
BUF;
			// So, merge buttons and select list
			$caption = ucfirst($this->caption);
			$code = <<< SPAN
<span>$caption ($articlesCount articles) :</span>\n
SPAN;
			// Firstly, 2 buttons
			$v = ($n > 0) ? $values[0] : false;
			$code .= TABPAGER2.$this->buildLinkArticle($v, '&lt;&lt;');
			$v = ($n > 0) ? $values[$n - 1] : false;
			$code .= TABPAGER2.$this->buildLinkArticle($v, '&lt;');
			// Now, the select list
			$code .= $buf;
			// Lastly, 2 more buttons
			if ($n !== false) {
				$v = ($n < count($values) - 1)? $values[$n + 1] : false;
				$code .= TABPAGER2.$this->buildLinkArticle($v, '&gt;');
			}
			$v = ($n < count($values) - 1)? $values[count($values)-1] : false;
			$code .= TABPAGER2.$this->buildLinkArticle($v, '&gt;&gt;');
		}
		else
			$code = '';
		return $code;
	}

	public function ThemeEndHead($params) {
		global $plxMotor;

		// add prev and next rel links in the head of html page
		if (in_array($plxMotor->mode, array('home', 'categorie', 'tags', 'archives'))) {
			// we have $articlesCount chapô or articles on the page
			$byPage = $plxMotor->bypage;
			$articlesCount = $plxMotor->plxGlob_arts->count;
			$totalPages = ceil($articlesCount / $byPage);
			$pageCC = $plxMotor->page;
			if ($pageCC > 1) { // previous link ?>
	<link rel="prev" href="<?php echo $this->urlPage($pageCC - 1)?>" />
<?php		}
			if ($pageCC < $totalPages - 1) { // next link ?>
	<link rel="next" href="<?php echo $this->urlPage($pageCC + 1)?>" />
<?php		}
		} elseif ($plxMotor->mode == 'article') {
			// just one article on the page. So, look for the previous and next articles
			$cible = $plxMotor->cible;
			if(!empty($this->articles)) {
				foreach ($this->articles as $k=>$v) {
					if (substr($v, 0, 4) == $cible) {
						if ($k > 0) { // previous link ?>
	<link rel="prev" href="<?php echo $this->urlArticle($this->articles[$k-1]); ?>" />
<?php					}
						if ($k < count($this->articles) - 1) { // next link ?>
	<link rel="next" href="<?php echo $this->urlArticle($this->articles[$k+1]); ?>" />
<?php					}
					break;
					}
				}
			}
		} elseif ($plxMotor->mode == 'static') {
			$pageCC = $this->staticPageCC;
			if ($pageCC > 1) { ?>
	<link rel="prev" href="<?php echo $this->urlStaticPage($pageCC - 1); ?>" />
<?php		}
			if ($pageCC < count($this->staticPagesGroup) - 1) { ?>
	<link rel="next" href="<?php echo $this->urlStaticPage($pageCC + 1); ?>" />
<?php		}
		}
		if (in_array($plxMotor->mode, array('article', 'home', 'categorie', 'tags', 'archives', 'static'))) {
			$this->add_scripts();
		}
	}

	// For displaying articlesNavigator, add in your template a new hook showArticlesBar or use ThemeEndBody hook by dfault
	public function showArticlesBar($params) {
		$this->usingShowArticlesHook = true; ?>
		<p id="articlesNavigator">
<?php echo $this->articlesNavigator(); ?>
		</p>
<?php
	}

	public function ThemeEndBody($params) {
		global $plxShow;

		if (! $this->usingShowArticlesHook and ($plxShow->mode() == 'article')) { ?>
	<div id="bottomNavBar">
		<p id="articlesNavigator">
<?php echo $this->articlesNavigator(); ?>
		</p>
	</div>
<?php
		}
	}
}
?>
